import { EventEmitter, NavigationModule, EVENT_WILL_SET_ROOT, EVENT_DID_SET_ROOT, EVENT_SWITCH_TAB, KEY_SCENE_ID, KEY_INDEX, RESULT_CANCEL, } from './NavigationModule';
import { bindBarButtonItemClickEvent } from './utils';
import store from './store';
let interceptor;
let shouldCallWillSetRootCallback = 0;
let willSetRootCallback;
let didSetRootCallback;
let tag = 0;
EventEmitter.addListener(EVENT_DID_SET_ROOT, _ => {
    didSetRootCallback && didSetRootCallback();
    shouldCallWillSetRootCallback = 0;
});
EventEmitter.addListener(EVENT_WILL_SET_ROOT, _ => {
    if (shouldCallWillSetRootCallback === 0 && willSetRootCallback) {
        willSetRootCallback();
    }
});
EventEmitter.addListener(EVENT_SWITCH_TAB, event => {
    const index = event[KEY_INDEX];
    const [from, to] = index.split('-');
    Navigator.dispatch(event[KEY_SCENE_ID], 'switchTab', {
        from: Number(from),
        to: Number(to),
    });
});
function checkRequestCode(reqCode) {
    if (isNaN(reqCode)) {
        return --tag;
    }
    if (reqCode < 0) {
        throw new Error('`requestCode` must be positive.');
    }
    return reqCode;
}
export class Navigator {
    constructor(sceneId, moduleName) {
        this.sceneId = sceneId;
        this.moduleName = moduleName;
        this.visibility = 'pending';
        this.state = {
            params: {},
            resultListeners: [],
        };
        this.dispatch = this.dispatch.bind(this);
        this.setParams = this.setParams.bind(this);
        this.push = this.push.bind(this);
        this.pop = this.pop.bind(this);
        this.popTo = this.popTo.bind(this);
        this.popToRoot = this.popToRoot.bind(this);
        this.redirectTo = this.redirectTo.bind(this);
        this.isStackRoot = this.isStackRoot.bind(this);
        this.present = this.present.bind(this);
        this.dismiss = this.dismiss.bind(this);
        this.showModal = this.showModal.bind(this);
        this.hideModal = this.hideModal.bind(this);
        this.setResult = this.setResult.bind(this);
        this.toggleMenu = this.toggleMenu.bind(this);
        this.openMenu = this.openMenu.bind(this);
        this.closeMenu = this.closeMenu.bind(this);
    }
    static of(sceneId) {
        let navigator = store.getNavigator(sceneId);
        if (!navigator) {
            navigator = new Navigator(sceneId);
            store.addNavigator(sceneId, navigator);
        }
        return navigator;
    }
    static async find(moduleName) {
        const sceneId = await NavigationModule.findSceneIdByModuleName(moduleName);
        if (sceneId) {
            return Navigator.of(sceneId);
        }
    }
    static async current() {
        const route = await Navigator.currentRoute();
        return Navigator.of(route.sceneId);
    }
    static async currentRoute() {
        return await NavigationModule.currentRoute();
    }
    static async routeGraph() {
        return await NavigationModule.routeGraph();
    }
    static setRoot(layout, sticky = false) {
        const pureLayout = bindBarButtonItemClickEvent(layout, {
            inLayout: true,
            navigatorFactory: (sceneId) => {
                return Navigator.of(sceneId);
            },
        });
        if (willSetRootCallback) {
            shouldCallWillSetRootCallback++;
            willSetRootCallback();
        }
        const flag = --tag;
        NavigationModule.setRoot(pureLayout, sticky, flag);
        return new Promise(resolve => {
            const subscription = EventEmitter.addListener(EVENT_DID_SET_ROOT, (data) => {
                if (data.tag === flag) {
                    subscription.remove();
                    resolve();
                    console.warn('react-native-navigation-hybrid was rename to hybrid-navigation, please use hybrid-navigation instead. see https://github.com/listenzz/hybrid-navigation');
                }
            });
        });
    }
    static setRootLayoutUpdateListener(willSetRoot = () => { }, didSetRoot = () => { }) {
        willSetRootCallback = willSetRoot;
        didSetRootCallback = didSetRoot;
    }
    static async dispatch(sceneId, action, params = {}) {
        let intercepted = false;
        if (interceptor) {
            const result = interceptor(action, {
                sceneId,
                ...params,
            });
            if (result instanceof Promise) {
                intercepted = await result;
            }
            else {
                intercepted = result;
            }
        }
        if (!intercepted) {
            return await NavigationModule.dispatch(sceneId, action, params);
        }
        return false;
    }
    static setInterceptor(interceptFn) {
        interceptor = interceptFn;
    }
    setParams(params) {
        this.state.params = { ...this.state.params, ...params };
    }
    dispatch(action, params = {}) {
        return Navigator.dispatch(this.sceneId, action, {
            from: this.moduleName,
            to: params.moduleName,
            ...params,
        });
    }
    result(requestCode, resultCode, data) {
        this.state.resultListeners.forEach(listener => {
            listener(requestCode, resultCode, data);
        });
    }
    unmount() {
        this.state.resultListeners.forEach(listener => {
            listener.cancel();
        });
        this.state.resultListeners.length = 0;
    }
    waitResult(requestCode, successful) {
        if (!successful) {
            return Promise.resolve([0, null]);
        }
        return new Promise(resolve => {
            const listener = (reqCode, resultCode, data) => {
                if (requestCode === reqCode) {
                    resolve([resultCode, data]);
                    const index = this.state.resultListeners.indexOf(listener);
                    if (index !== -1) {
                        this.state.resultListeners.splice(index, 1);
                    }
                }
            };
            listener.cancel = () => {
                resolve([RESULT_CANCEL, null]);
            };
            this.state.resultListeners.push(listener);
        });
    }
    async push(moduleName, props = {}, options = {}) {
        const success = await this.dispatch('push', { moduleName, props, options });
        return await this.waitResult(0, success);
    }
    async pushLayout(layout) {
        const success = await this.dispatch('pushLayout', { layout });
        return await this.waitResult(0, success);
    }
    pop() {
        return this.dispatch('pop');
    }
    popTo(moduleName) {
        return this.dispatch('popTo', { moduleName });
    }
    popToRoot() {
        return this.dispatch('popToRoot');
    }
    redirectTo(moduleName, props = {}, options = {}) {
        return this.dispatch('redirectTo', {
            moduleName,
            props,
            options,
            animated: true,
        });
    }
    isStackRoot() {
        return NavigationModule.isNavigationRoot(this.sceneId);
    }
    async present(moduleName, props = {}, options = {}, requestCode = NaN) {
        requestCode = checkRequestCode(requestCode);
        const success = await this.dispatch('present', {
            moduleName,
            props,
            options,
            requestCode,
        });
        return await this.waitResult(requestCode, success);
    }
    async presentLayout(layout, requestCode = NaN) {
        requestCode = checkRequestCode(requestCode);
        const success = await this.dispatch('presentLayout', { layout, requestCode });
        return await this.waitResult(requestCode, success);
    }
    dismiss() {
        return this.dispatch('dismiss');
    }
    async showModal(moduleName, props = {}, options = {}, requestCode = NaN) {
        requestCode = checkRequestCode(requestCode);
        const success = await this.dispatch('showModal', {
            moduleName,
            props,
            options,
            requestCode,
        });
        return await this.waitResult(requestCode, success);
    }
    async showModalLayout(layout, requestCode = NaN) {
        requestCode = checkRequestCode(requestCode);
        const success = await this.dispatch('showModalLayout', { layout, requestCode });
        return await this.waitResult(requestCode, success);
    }
    hideModal() {
        return this.dispatch('hideModal');
    }
    setResult(resultCode, data = null) {
        NavigationModule.setResult(this.sceneId, resultCode, data);
    }
    async switchTab(index, popToRoot = false) {
        const from = await NavigationModule.currentTab(this.sceneId);
        return this.dispatch('switchTab', { from, to: index, popToRoot });
    }
    toggleMenu() {
        return this.dispatch('toggleMenu');
    }
    openMenu() {
        return this.dispatch('openMenu');
    }
    closeMenu() {
        return this.dispatch('closeMenu');
    }
    signalFirstRenderComplete() {
        NavigationModule.signalFirstRenderComplete(this.sceneId);
    }
}
